@@ -39,11 +39,24 @@ from .plugins.settings import (
     save_state_yaml,
     validate_config,
 )
-from .utils import expanduser, fix_perms, get_context, get_os, switch_priviledge
+from .utils import (
+    GIT_ADJ,
+    GIT_HHD,
+    HHD_DEV_DIR,
+    expanduser,
+    get_ac_status,
+    get_ac_status_fn,
+    get_context,
+    get_os,
+    switch_priviledge,
+    refresh_is_steam_running,
+)
 
 logger = logging.getLogger(__name__)
 
-CONFIG_DIR = os.environ.get("HHD_CONFIG_DIR", "~/.config/hhd")
+CONFIG_DIR = os.environ.get("HHD_CONFIG_DIR", "/etc/hhd")
+TOKEN_FILE = os.environ.get("HHD_TOKEN_FILE", "/tmp/hhd/token")
+PLUGIN_WHITELIST = os.environ.get("HHD_PLUGINS", "")
 
 ERROR_DELAY = 5
 INIT_DELAY = 0.4
@@ -52,10 +65,10 @@ SLEEP_MIN_T = 8
 
 
 class EmitHolder(Emitter):
-    def __init__(self, condition: Condition, ctx) -> None:
+    def __init__(self, condition: Condition, ctx, info) -> None:
         self._events = []
         self._condition = condition
-        super().__init__(ctx=ctx)
+        super().__init__(ctx=ctx, info=info)
 
     def __call__(self, event: Event | Sequence[Event]) -> None:
         with self._condition:
@@ -96,20 +109,13 @@ def get_wakeup_count():
 
 
 def print_token(ctx):
-    token_fn = expanduser(join(CONFIG_DIR, "token"), ctx)
-
     try:
-        with open(token_fn, "r") as f:
+        with open(TOKEN_FILE, "r") as f:
             token = f.read().strip()
 
         logger.info(f'Current HHD token (for user "{ctx.name}") is: "{token}"')
     except Exception as e:
         logger.error(f"Token not found or could not be read, error:\n{e}")
-        logger.info(
-            "Enable the http endpoint to generate a token automatically.\n"
-            + "Or place it under '~/.config/hhd/token' manually.\n"
-            + "'chown 600 ~/.config/hhd/token' for security reasons!"
-        )
 
 
 def main():
@@ -121,7 +127,7 @@ def main():
         "-u",
         "--user",
         default=None,
-        help="The user whose home directory will be used to store the files (~/.config/hhd).",
+        help="This argument has been deprecated. Handheld Daemon will always use /etc/hhd moving forward.",
         dest="user",
     )
     parser.add_argument(
@@ -134,21 +140,25 @@ def main():
     user = args.user
 
     # Setup temporary logger for permission retrieval
-    ctx = get_context(user)
-    if not ctx:
-        print(f"Could not get user information. Exiting...")
-        return
+    if user:
+        ctx = get_context(user)
+    else:
+        ctx = None
 
     detectors: dict[str, HHDAutodetect] = {}
     plugins: dict[str, Sequence[HHDPlugin]] = {}
     cfg_fds = []
+    switch_root = None
 
     # HTTP data
     https = None
     prev_http_cfg = None
     updated = False
     last_event = None
+    ac_fn = get_ac_status_fn()
     info = Config()
+    info["ac"] = None
+    ac_status = None
 
     # Check we are in a virtual environment
     # TODO: Improve
@@ -158,20 +168,9 @@ def main():
         # Create nested hhd dir
         # This might mess up permissions in upward directories
         # So try to deescalate
-        hhd_dir = expanduser(CONFIG_DIR, ctx)
+        hhd_dir = CONFIG_DIR
         try:
-            switch_priviledge(ctx, False)
             os.makedirs(hhd_dir, exist_ok=True)
-            switch_priviledge(ctx, True)
-            fix_perms(hhd_dir, ctx)
-        except Exception:
-            pass
-
-        # Remove old dir
-        try:
-            os.rename(
-                join(hhd_dir, "plugins"), join(hhd_dir, "plugins_old_USE_STATEYML")
-            )
         except Exception:
             pass
 
@@ -185,6 +184,45 @@ def main():
             else:
                 logger.error(f"Command '{args.command[0]}' is unknown. Ignoring...")
 
+        # Run a one time migration
+        # TODO: remove this e.g., after a year
+        try:
+            migrate_fn = join(hhd_dir, ".migrated")
+            state_fn = join(CONFIG_DIR, "state.yml")
+            if not os.path.isfile(migrate_fn) and not os.path.isfile(state_fn):
+                # Always touch migration file so we only run on a fresh install
+                with open(migrate_fn, "w") as f:
+                    f.write("")
+
+                for user in os.listdir("/home"):
+                    old_state = expanduser("~/.config/hhd/state.yml", ctx)
+                    if os.path.isfile(old_state):
+                        import shutil
+
+                        shutil.copy(old_state, state_fn)
+                        os.chmod(state_fn, 0o644)
+                        os.chown(state_fn, 0, 0)
+                        logger.warning(
+                            f"Migrated state file {old_state} to {state_fn}."
+                        )
+                        break
+        except Exception as e:
+            logger.error(f"Could not migrate state file. Error:\n{e}")
+
+        # Lock a file to ensure hhd is only running once, or wait to unlock
+        # THis is for legacy enabled services with --user
+        lock_fn = join(hhd_dir, "lock")
+        lock_fd = -1
+        if os.environ.get("HHD_SWITCH_ROOT", None) != "1":
+            try:
+                logger.info("Trying to acquire hhd lock...")
+                lock_fd = os.open(lock_fn, os.O_RDWR | os.O_CREAT)
+                fcntl.flock(lock_fd, fcntl.LOCK_EX)
+            except Exception as e:
+                logger.error(
+                    f"Could not acquire lock for hhd.\nError:\n{e}"
+                )
+
         # Get OS Info
         info["os"] = get_os()
 
@@ -195,17 +233,20 @@ def main():
         logger.info(f"Running autodetection...")
 
         detector_names = []
+        whitelist = PLUGIN_WHITELIST.split(",") if PLUGIN_WHITELIST else []
         for autodetect in pkg_resources.iter_entry_points("hhd.plugins"):
             name = autodetect.name
             detector_names.append(name)
             if name in blacklist:
                 logger.info(f"Skipping blacklisted provider '{name}'.")
-            else:
-                detectors[autodetect.name] = autodetect.resolve()
+            if whitelist and name not in whitelist:
+                logger.info(f"Skipping provider '{name}' due to whitelist.")
+                continue
+
+            detectors[autodetect.name] = autodetect.resolve()
 
         # Save new blacklist file
         save_blacklist_yaml(blacklist_fn, detector_names, blacklist)
-        fix_perms(blacklist_fn, ctx)
 
         logger.info(f"Found plugin providers: {', '.join(list(detectors))}")
 
@@ -253,16 +294,16 @@ def main():
         # Open plugins
         lock = RLock()
         cond = Condition(lock)
-        emit = EmitHolder(cond, ctx)
+        emit = EmitHolder(cond, ctx, info)
         for p in sorted_plugins:
             set_log_plugin(getattr(p, "log") if hasattr(p, "log") else "ukwn")
-            p.open(emit, ctx)
+            p.open(emit, ctx) # type: ignore
             update_log_plugins()
         set_log_plugin("main")
 
         # Compile initial configuration
-        state_fn = expanduser(join(CONFIG_DIR, "state.yml"), ctx)
-        token_fn = expanduser(join(CONFIG_DIR, "token"), ctx)
+        state_fn = join(CONFIG_DIR, "state.yml")
+        token_fn = join(CONFIG_DIR, ".token")
         settings: HHDSettings = {}
         shash = None
 
@@ -270,9 +311,8 @@ def main():
         profiles = {}
         templates = {}
         conf = Config({})
-        profile_dir = expanduser(join(CONFIG_DIR, "profiles"), ctx)
+        profile_dir = join(CONFIG_DIR, "profiles")
         os.makedirs(profile_dir, exist_ok=True)
-        fix_perms(profile_dir, ctx)
 
         # Monitor config files for changes
         should_initialize = TEvent()
@@ -310,7 +350,7 @@ def main():
                 except Exception as e:
                     logger.warning(f"Could not hide update settings. Error:\n{e}")
                 settings = merge_settings(
-                    [*[p.settings() for p in sorted_plugins], hhd_settings]
+                    [hhd_settings, *[p.settings() for p in sorted_plugins]]
                 )
                 # Force general settings to be last
                 if "hhd" in settings:
@@ -361,7 +401,7 @@ def main():
 
                     exe = find_overlay_exe(ctx)
                     if exe:
-                        ver = get_overlay_version(exe, ctx)
+                        ver = get_overlay_version(exe)
                         conf["hhd.settings.version_ui"] = ver
                         logger.info(f"Overlay Version: {ver}")
                     else:
@@ -374,7 +414,6 @@ def main():
                 profiles = {}
                 templates = {}
                 os.makedirs(profile_dir, exist_ok=True)
-                fix_perms(profile_dir, ctx)
                 for fn in os.listdir(profile_dir):
                     if not fn.endswith(".yml"):
                         continue
@@ -409,16 +448,22 @@ def main():
                     join(CONFIG_DIR, "profiles"),
                 ]
                 for fn in cfg_fns:
-                    fd = os.open(expanduser(fn, ctx), os.O_RDONLY)
-                    fcntl.fcntl(
-                        fd,
-                        fcntl.F_NOTIFY,
-                        fcntl.DN_CREATE
-                        | fcntl.DN_DELETE
-                        | fcntl.DN_MODIFY
-                        | fcntl.DN_RENAME
-                        | fcntl.DN_MULTISHOT,
-                    )
+                    fd = -1
+                    try:
+                        fd = os.open(fn, os.O_RDONLY)
+                        fcntl.fcntl(
+                            fd,
+                            fcntl.F_NOTIFY,
+                            fcntl.DN_CREATE
+                            | fcntl.DN_DELETE
+                            | fcntl.DN_MODIFY
+                            | fcntl.DN_RENAME
+                            | fcntl.DN_MULTISHOT,
+                        )
+                    except Exception:
+                        if fd != -1:
+                            os.close(fd)
+                        continue
                     cfg_fds.append(fd)
 
                 should_initialize.clear()
@@ -430,46 +475,53 @@ def main():
                 prev_http_cfg = http_cfg
                 if https:
                     https.close()
-                if http_cfg["enable"].to(bool):
-                    from .http import HHDHTTPServer
 
-                    port = http_cfg["port"].to(int)
-                    localhost = http_cfg["localhost"].to(bool)
-                    use_token = http_cfg["token"].to(bool)
+                from .http import HHDHTTPServer
 
+                port = 5335
+                localhost = http_cfg["localhost"].to(bool)
+                use_token = http_cfg["token"].to(bool)
+
+                if use_token:
                     # Generate security token
-                    if use_token:
-                        if not os.path.isfile(token_fn):
-                            import hashlib
-                            import random
+                    if not os.path.isfile(token_fn):
+                        import hashlib
+                        import random
 
-                            token = hashlib.sha256(
-                                str(random.random()).encode()
-                            ).hexdigest()[:12]
-                            with open(token_fn, "w") as f:
-                                os.chmod(token_fn, 0o600)
-                                f.write(token)
-                            fix_perms(token_fn, ctx)
-                        else:
-                            with open(token_fn, "r") as f:
-                                token = f.read().strip()
+                        token = hashlib.sha256(
+                            str(random.random()).encode()
+                        ).hexdigest()[:12]
+                        with open(token_fn, "w") as f:
+                            os.chmod(token_fn, 0o644)
+                            f.write(token)
                     else:
-                        token = None
+                        with open(token_fn, "r") as f:
+                            token = f.read().strip()
+                    
+                    # Save the token to an ephemeral dir
+                    os.makedirs(
+                        os.path.dirname(TOKEN_FILE), exist_ok=True
+                    )
+                    with open(TOKEN_FILE, "w") as f:
+                        os.chmod(TOKEN_FILE, 0o644)
+                        f.write(token)
+                else:
+                    token = None
 
-                    set_log_plugin("rest")
-                    https = HHDHTTPServer(localhost, port, token)
-                    https.update(settings, conf, info, profiles, emit, locales, ctx)
-                    try:
-                        https.open()
-                    except Exception as e:
-                        logger.error(
-                            f"Could not start http API on port {port}.\n"
-                            + "Is another version of Handheld Daemon open?\n"
-                            + "Closing."
-                        )
-                        return
-                    update_log_plugins()
-                    set_log_plugin("main")
+                set_log_plugin("rest")
+                https = HHDHTTPServer(localhost, port, token)
+                https.update(settings, conf, info, profiles, emit, locales)
+                try:
+                    https.open()
+                except Exception as e:
+                    logger.error(
+                        f"Could not start http API on port {port}.\n"
+                        + "Is another version of Handheld Daemon open?\n"
+                        + "Closing."
+                    )
+                    return
+                update_log_plugins()
+                set_log_plugin("main")
 
             #
             # Plugin loop
@@ -506,6 +558,14 @@ def main():
             wakeup_count = new_wakeup_count
             last_event = curr
 
+            # AC status
+            if ac_fn:
+                new_status = get_ac_status(ac_fn)
+                if new_status != ac_status:
+                    logger.info(f"AC status is: {new_status}")
+                    ac_status = new_status
+                    info["ac"] = ac_status
+
             for ev in events:
                 match ev["type"]:
                     case "settings":
@@ -530,17 +590,33 @@ def main():
                             conf.update(profiles[ev["name"]].conf)
                     case "state":
                         conf.update(ev["config"].conf)
-                    case "special" | "acpi" | "tdp" | "ppd" | "energy":
+                    case "special":
+                        if ev["event"] == "restart_dev":
+                            should_exit.set()
+                            switch_root = True
+                            break
+                        elif ev["event"] == "shutdown_dev":
+                            should_exit.set()
+                            # Trigger restart
+                            updated = True
+                    case "acpi" | "tdp" | "ppd" | "energy":
                         pass
                     case other:
                         logger.error(f"Invalid event type submitted: '{other}'")
 
+            # Refresh steam status
+            # TODO: Move this?
+            refresh_is_steam_running()
+
             # If settings changed, the configuration needs to reload
             # but it needs to be saved first
             if settings_changed:
                 logger.info(f"Reloading settings.")
 
                 # Settings
+                settings_base = {
+                    k: {} for k in load_relative_yaml("sections.yml")["sections"]
+                }
                 hhd_settings = {"hhd": load_relative_yaml("settings.yml")}
                 # TODO: Improve check
                 try:
@@ -550,7 +626,11 @@ def main():
                 except Exception as e:
                     logger.warning(f"Could not hide update settings. Error:\n{e}")
                 settings = merge_settings(
-                    [*[p.settings() for p in sorted_plugins], hhd_settings]
+                    [
+                        settings_base,
+                        hhd_settings,
+                        *[p.settings() for p in sorted_plugins],
+                    ]
                 )
                 # Force general settings to be last
                 if "hhd" in settings:
@@ -593,7 +673,7 @@ def main():
             # Notify that events were applied
             # Before saving to reduce delay (yaml files take 100ms :( )
             if https:
-                https.update(settings, conf, info, profiles, emit, locales, ctx)
+                https.update(settings, conf, info, profiles, emit, locales)
 
             #
             # Save loop
@@ -603,13 +683,11 @@ def main():
             saved = False
             # Save existing profiles if open
             if save_state_yaml(state_fn, settings, conf, shash):
-                fix_perms(state_fn, ctx)
                 saved = True
                 conf.updated = False
             for name, prof in profiles.items():
                 fn = join(profile_dir, name + ".yml")
                 if save_profile_yaml(fn, settings, prof, shash):
-                    fix_perms(fn, ctx)
                     saved = True
                     prof.updated = False
             for prof in os.listdir(profile_dir):
@@ -627,17 +705,6 @@ def main():
                             f"Failed removing profile {name} at:\n{fn}\nWith error:\n{e}"
                         )
 
-            # Causes unnecessary writes, is not used anyway.
-            # # Add template config
-            # if save_profile_yaml(
-            #     join(profile_dir, "_template.yml"),
-            #     settings,
-            #     templates.get("_template", None),
-            #     shash,
-            # ):
-            #     fix_perms(join(profile_dir, "_template.yml"), ctx)
-            #     saved = True
-
             if not has_new and saved:
                 # We triggered the interrupt, clear
                 should_initialize.clear()
@@ -650,7 +717,8 @@ def main():
                 conf["hhd.settings.update_stable"] = False
                 conf["hhd.settings.update_beta"] = False
 
-                switch_priviledge(ctx, False)
+                if ctx:
+                    switch_priviledge(ctx.uid)
                 try:
                     logger.info(f"Updating Handheld Daemon.")
                     if "venv" in exe_python:
@@ -674,16 +742,8 @@ def main():
                                 "--upgrade",
                                 "--cache-dir",
                                 "/tmp/__hhd_update_cache",
-                                (
-                                    "git+https://github.com/hhd-dev/hhd"
-                                    if upd_beta
-                                    else "hhd"
-                                ),
-                                (
-                                    "git+https://github.com/hhd-dev/adjustor"
-                                    if upd_beta
-                                    else "adjustor"
-                                ),
+                                (GIT_HHD if upd_beta else "hhd"),
+                                (GIT_ADJ if upd_beta else "adjustor"),
                             ]
                         )
 
@@ -735,7 +795,7 @@ def main():
                     err = f"Error while updating:\n{e}"
                     conf["hhd.settings.update_error"] = err
                     logger.error(err)
-                switch_priviledge(ctx, True)
+                switch_priviledge(0)
 
                 if updated:
                     should_exit.set()
@@ -780,9 +840,11 @@ def main():
             logger.info("Closing cached controllers.")
             from hhd.controller.virtual.dualsense import Dualsense
             from hhd.controller.virtual.uinput import UInputDevice
+            from hhd.controller.virtual.sd import SteamdeckController
 
             UInputDevice.close_cached()
             Dualsense.close_cached()
+            SteamdeckController.close_cached()
         except Exception as e:
             logger.error("Could not close cached controllers with error:\n{e}")
 
@@ -790,6 +852,11 @@ def main():
         # Use error code to restart service
         sys.exit(-1)
 
+    if switch_root:
+        os.environ["HHD_SWITCH_ROOT"] = "1"
+        o = subprocess.run([HHD_DEV_DIR + "/bin/hhd", *sys.argv], check=False)
+        sys.exit(o.returncode)
+
 
 if __name__ == "__main__":
     main()
