@@ -69,6 +69,7 @@ CONFS = {
     # Aokzoe
     "AOKZOE A1 AR07": AOKZOE_CONF,
     "AOKZOE A1 Pro": AOKZOE_CONF,
+    "AOKZOE A2 Pro": AOKZOE_CONF,
     "AOKZOE A1X": {
         "name": "AOKZOE A1X",
         "hrtimer": True,
@@ -121,6 +122,14 @@ CONFS = {
         "mapping": X1_MAPPING,
         "protocol": "serial",
     },
+    "ONEXPLAYER X1z": {
+        **ONEX_DEFAULT_CONF,
+        "name": "ONEXPLAYER X1 (AMD)",
+        "x1": True,
+        "rgb_secondary": True,
+        "mapping": X1_MAPPING,
+        "protocol": "serial",
+    },
     "ONEXPLAYER X1Pro": {
         **ONEX_DEFAULT_CONF,
         "name": "ONEXPLAYER X1 Pro (AMD)",
@@ -147,6 +156,15 @@ CONFS = {
         "protocol": "serial",
         "turbo": False,  # disable turbo takeover so that it can be used for TDP
     },
+    "ONEXPLAYER X1Air": {
+        **ONEX_DEFAULT_CONF,
+        "name": "ONEXPLAYER X1 (Intel)",
+        "x1": True,
+        "rgb_secondary": True,
+        "mapping": X1_MAPPING,
+        "protocol": "hid_v1",
+        "turbo": False,  # disable turbo takeover so that it can be used for TDP
+    },
     "ONEXPLAYER G1 i": {
         **ONEX_DEFAULT_CONF,
         "name": "ONEXPLAYER G1 (Intel)",
@@ -179,5 +197,6 @@ def get_default_config(product_name: str, manufacturer: str):
     if "aokzoe" in manufacturer.lower():
         out["protocol"] = "none"
         out["rgb"] = False
+        out["aok"] = True
 
     return out
